@@ -1,5 +1,5 @@
-# Copyright (c) 2012 Mitch Garnaat http://garnaat.org/
-# Copyright 2012 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright (c) 2012-2013 Mitch Garnaat http://garnaat.org/
+# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the
@@ -22,6 +22,7 @@
 #
 from .endpoint import get_endpoint
 from .operation import Operation
+from .exceptions import ServiceNotInRegionError, NoRegionError
 
 
 class Service(object):
@@ -39,14 +40,15 @@ class Service(object):
 
     def __init__(self, session, provider_name, service_name,
                  path='/', port=None):
-        self.membered_lists = True
+        self.membered_lists = False
+        self.global_endpoint = None
         sdata = session.get_service_data(service_name, provider_name)
         self.__dict__.update(sdata)
         self.session = session
         self.provider_name = provider_name
         self.path = path
         self.port = port
-        self.cli_name = self.short_name
+        self.cli_name = service_name
         self._operations_data = self.operations
         self.operations = []
         for operation_data in self._operations_data:
@@ -55,13 +57,26 @@ class Service(object):
             setattr(self, op.py_name, op)
 
     def __repr__(self):
-        return 'Service(%s)' % self.name
+        return 'Service(%s)' % self.service_abbreviation
 
     @property
     def region_names(self):
         return self.regions.keys()
 
-    def get_endpoint(self, region_name, is_secure=True, endpoint_url=None):
+    def build_endpoint_url(self, host, is_secure):
+        if is_secure:
+            scheme = 'https'
+        else:
+            scheme = 'http'
+        if scheme not in self.protocols:
+            raise ValueError('Unsupported protocol: %s' % scheme)
+        endpoint_url = '%s://%s%s' % (scheme, host, self.path)
+        if self.port:
+            endpoint_url += ':%d' % self.port
+        return endpoint_url
+
+    def get_endpoint(self, region_name=None, is_secure=True,
+                     endpoint_url=None):
         """
         Return the Endpoint object for this service in a particular
         region.
@@ -71,22 +86,28 @@ class Service(object):
 
         :type is_secure: bool
         :param is_secure: True if you want the secure (HTTPS) endpoint.
+
+        :type endpoint_url: str
+        :param endpoint_url: You can explicitly override the default
+            computed endpoint name with this parameter.
         """
+        if region_name is None:
+            if self.global_endpoint and not self.session.profile:
+                endpoint_url = self.build_endpoint_url(self.global_endpoint,
+                                                       is_secure)
+                region_name = 'us-east-1'
+            else:
+                region_name = self.session.get_variable('region')
+        if region_name is None:
+            envvar_name = self.session.env_vars['region'][1]
+            raise NoRegionError(env_var=envvar_name)
         if region_name not in self.regions:
-            raise ValueError('Service: %s not available in region: %s' %
-                             (self.service_name, region_name))
+            raise ServiceNotInRegionError(service_name=self.endpoint_prefix,
+                                          region_name=region_name)
         endpoint_url = endpoint_url or self.regions[region_name]
         if endpoint_url is None:
-            if is_secure:
-                scheme = 'https'
-            else:
-                scheme = 'http'
-            if scheme not in self.protocols:
-                raise ValueError('Unsupported protocol: %s' % scheme)
-            host = '%s.%s.amazonaws.com' % (self.short_name, region_name)
-            endpoint_url = '%s://%s%s' % (scheme, host, self.path)
-            if self.port:
-                endpoint_url += ':%d' % self.port
+            host = '%s.%s.amazonaws.com' % (self.endpoint_prefix, region_name)
+            endpoint_url = self.build_endpoint_url(host, is_secure)
         return get_endpoint(self, region_name, endpoint_url)
 
     def get_operation(self, operation_name):
